<div class="peek-nav-section">
    <!--
        The following 'div' groups button to the left of the Nav Bar.
        Can contain one to many buttons
    <div class="btn-group" role="group">
        <Button class="nav-section-btn"
                role="group"
                (click)="navToDiagram()">
            Open GIS Diagram
        </Button>
    </div>
    -->
</div>

<div class="peek-nav-bar-padding">
    <div class="peek-information-section">
        <p>
            Grid Cache Offline Mode Enabled :
            {{gridLoaderStatus.cacheForOfflineEnabled}}<br />
            Grid Cache Load Complete :
            {{gridLoaderStatus.initialLoadComplete}}<br />
            Grid Cache Load Progress :
            {{gridLoaderStatus.loadProgress}}/{{gridLoaderStatus.loadTotal}}<br />
            Grid Cache Last Check Finished : {{gridLoaderStatus.lastCheck}}<br />

            Location Index Offline Mode Enabled :
            {{locationLoaderStatus.cacheForOfflineEnabled}}<br />
            Location Index Load Complete :
            {{locationLoaderStatus.initialLoadComplete}}<br />
            Location Index Load Progress :
            {{locationLoaderStatus.loadProgress}}/{{locationLoaderStatus.loadTotal}}<br />
            Location Index Last Check : {{locationLoaderStatus.lastCheck}}<br />
        </p>
    </div>
</div>
