export enum EditorToolType {
    NONE,
    SELECT_TOOL,
    EDIT_SELECT_TOOL,
    EDIT_MAKE_TEXT,
    EDIT_MAKE_RECTANGLE,
    EDIT_MAKE_LINE_WITH_ARROW,
    EDIT_MAKE_POLYGON,
    EDIT_MAKE_POLYLINE,
    EDIT_MAKE_CIRCLE_ELLIPSE_ARC,
    EDIT_MAKE_DISP_GROUP_PTR_VERTEX,
    EDIT_MAKE_DISP_POLYLINE_EDGE,
}
