@import "~@synerty/peek-plugin-base-js/src/styles/_mixins";
@import "~@synerty/peek-plugin-base-js/src/styles/_variables";

$height: calc(100vh - 300px);

.modal-open.modal {
    overflow-y: unset;
}

.modal-body {
    overflow-y: auto;
    max-height: 85vh;
}

.modal-dialog {
    padding: $general_padding_1;
    top: $title_bar_height;
}

.filter-div {
    display: flex;
    flex-direction: row;
}

nz-switch {
    padding-top: $general_padding_1;
    padding-left: $general_padding_1;
}

.scroll-outer {
    max-height: $height;
    height: $height;
    overflow-y: scroll;
}

/*
.modal-backdrop {
  float: left;
  top: 0px;
  bottom:0px;
  left:0px;
  right:0px;
  background-color: $primary_dark;
}
*/

.bar-left {
    border-left: 2px solid LightGray;
}
